#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "ADT.h"

/*{{{  conversion functions */

ATerm ADT_stringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

ATerm ADT_byteToChar(char ch)
{
    return (ATerm) ATmakeInt(ch);
}

char *ADT_charsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}

char ADT_charToByte(ATerm arg)
{
    return (char) ATgetInt((ATermInt) arg);
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _ADT_Entries;
typedef struct ATerm _ADT_Entry;
typedef struct ATerm _ADT_Separators;
typedef struct ATerm _ADT_Separator;

/*}}}  */

/*{{{  void ADT_initADTApi(void) */

void ADT_initADTApi(void)
{
  init_ADT_dict();
}

/*}}}  */

/*{{{  (un)protect functions */

void ADT_protectEntries(ADT_Entries *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectEntries(ADT_Entries *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectEntry(ADT_Entry *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectEntry(ADT_Entry *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectSeparators(ADT_Separators *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectSeparators(ADT_Separators *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectSeparator(ADT_Separator *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectSeparator(ADT_Separator *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  ADT_Entries ADT_EntriesFromTerm(ATerm t) */

ADT_Entries ADT_EntriesFromTerm(ATerm t)
{
  return (ADT_Entries)t;
}

/*}}}  */
/*{{{  ATerm ADT_EntriesToTerm(ADT_Entries arg) */

ATerm ADT_EntriesToTerm(ADT_Entries arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Entry ADT_EntryFromTerm(ATerm t) */

ADT_Entry ADT_EntryFromTerm(ATerm t)
{
  return (ADT_Entry)t;
}

/*}}}  */
/*{{{  ATerm ADT_EntryToTerm(ADT_Entry arg) */

ATerm ADT_EntryToTerm(ADT_Entry arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Separators ADT_SeparatorsFromTerm(ATerm t) */

ADT_Separators ADT_SeparatorsFromTerm(ATerm t)
{
  return (ADT_Separators)t;
}

/*}}}  */
/*{{{  ATerm ADT_SeparatorsToTerm(ADT_Separators arg) */

ATerm ADT_SeparatorsToTerm(ADT_Separators arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Separator ADT_SeparatorFromTerm(ATerm t) */

ADT_Separator ADT_SeparatorFromTerm(ATerm t)
{
  return (ADT_Separator)t;
}

/*}}}  */
/*{{{  ATerm ADT_SeparatorToTerm(ADT_Separator arg) */

ATerm ADT_SeparatorToTerm(ADT_Separator arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */

int ADT_getEntriesLength (ADT_Entries arg) {
  return ATgetLength((ATermList) arg);
}
ADT_Entries ADT_reverseEntries(ADT_Entries arg) {
  return (ADT_Entries) ATreverse((ATermList) arg);
}
ADT_Entries ADT_appendEntries(ADT_Entries arg, ADT_Entry elem) {
  return (ADT_Entries) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
ADT_Entries ADT_concatEntries(ADT_Entries arg0, ADT_Entries arg1) {
  return (ADT_Entries) ATconcat((ATermList) arg0, (ATermList) arg1);
}
ADT_Entries ADT_sliceEntries(ADT_Entries arg, int start, int end) {
  return (ADT_Entries) ATgetSlice((ATermList) arg, start, end);
}
ADT_Entry ADT_getEntriesEntryAt(ADT_Entries arg, int index) {
 return (ADT_Entry)ATelementAt((ATermList) arg,index);
}
ADT_Entries ADT_replaceEntriesEntryAt(ADT_Entries arg, ADT_Entry elem, int index) {
 return (ADT_Entries) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
ADT_Entries ADT_makeEntries2(ADT_Entry elem1, ADT_Entry elem2) {
  return (ADT_Entries) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
ADT_Entries ADT_makeEntries3(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3) {
  return (ADT_Entries) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
ADT_Entries ADT_makeEntries4(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4) {
  return (ADT_Entries) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
ADT_Entries ADT_makeEntries5(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4, ADT_Entry elem5) {
  return (ADT_Entries) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
ADT_Entries ADT_makeEntries6(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4, ADT_Entry elem5, ADT_Entry elem6) {
  return (ADT_Entries) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int ADT_getSeparatorsLength (ADT_Separators arg) {
  return ATgetLength((ATermList) arg);
}
ADT_Separators ADT_reverseSeparators(ADT_Separators arg) {
  return (ADT_Separators) ATreverse((ATermList) arg);
}
ADT_Separators ADT_appendSeparators(ADT_Separators arg, ADT_Separator elem) {
  return (ADT_Separators) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
ADT_Separators ADT_concatSeparators(ADT_Separators arg0, ADT_Separators arg1) {
  return (ADT_Separators) ATconcat((ATermList) arg0, (ATermList) arg1);
}
ADT_Separators ADT_sliceSeparators(ADT_Separators arg, int start, int end) {
  return (ADT_Separators) ATgetSlice((ATermList) arg, start, end);
}
ADT_Separator ADT_getSeparatorsSeparatorAt(ADT_Separators arg, int index) {
 return (ADT_Separator)ATelementAt((ATermList) arg,index);
}
ADT_Separators ADT_replaceSeparatorsSeparatorAt(ADT_Separators arg, ADT_Separator elem, int index) {
 return (ADT_Separators) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
ADT_Separators ADT_makeSeparators2(ADT_Separator elem1, ADT_Separator elem2) {
  return (ADT_Separators) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
ADT_Separators ADT_makeSeparators3(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3) {
  return (ADT_Separators) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
ADT_Separators ADT_makeSeparators4(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4) {
  return (ADT_Separators) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
ADT_Separators ADT_makeSeparators5(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4, ADT_Separator elem5) {
  return (ADT_Separators) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
ADT_Separators ADT_makeSeparators6(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4, ADT_Separator elem5, ADT_Separator elem6) {
  return (ADT_Separators) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}

/*}}}  */
/*{{{  constructors */

/*{{{  ADT_Entries ADT_makeEntriesEmpty(void) */

ADT_Entries ADT_makeEntriesEmpty(void)
{
  return (ADT_Entries)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT_Entries ADT_makeEntriesSingle(ADT_Entry head) */

ADT_Entries ADT_makeEntriesSingle(ADT_Entry head)
{
  return (ADT_Entries)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  ADT_Entries ADT_makeEntriesMany(ADT_Entry head, ADT_Entries tail) */

ADT_Entries ADT_makeEntriesMany(ADT_Entry head, ADT_Entries tail)
{
  return (ADT_Entries)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT_Entry ADT_makeEntryConstructor(ATerm sort, ATerm alternative, ATerm termPattern) */

ADT_Entry ADT_makeEntryConstructor(ATerm sort, ATerm alternative, ATerm termPattern)
{
  return (ADT_Entry)(ATerm)ATmakeAppl3(ADT_afun0, (ATerm) sort, (ATerm) alternative, (ATerm) termPattern);
}

/*}}}  */
/*{{{  ADT_Entry ADT_makeEntryList(ATerm sort, ATerm elemSort) */

ADT_Entry ADT_makeEntryList(ATerm sort, ATerm elemSort)
{
  return (ADT_Entry)(ATerm)ATmakeAppl2(ADT_afun1, (ATerm) sort, (ATerm) elemSort);
}

/*}}}  */
/*{{{  ADT_Entry ADT_makeEntrySeparatedList(ATerm sort, ATerm elemSort, ADT_Separators separators) */

ADT_Entry ADT_makeEntrySeparatedList(ATerm sort, ATerm elemSort, ADT_Separators separators)
{
  return (ADT_Entry)(ATerm)ATmakeAppl3(ADT_afun2, (ATerm) sort, (ATerm) elemSort, (ATerm) separators);
}

/*}}}  */
/*{{{  ADT_Separators ADT_makeSeparatorsEmpty(void) */

ADT_Separators ADT_makeSeparatorsEmpty(void)
{
  return (ADT_Separators)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT_Separators ADT_makeSeparatorsSingle(ADT_Separator head) */

ADT_Separators ADT_makeSeparatorsSingle(ADT_Separator head)
{
  return (ADT_Separators)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  ADT_Separators ADT_makeSeparatorsMany(ADT_Separator head, ADT_Separators tail) */

ADT_Separators ADT_makeSeparatorsMany(ADT_Separator head, ADT_Separators tail)
{
  return (ADT_Separators)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT_Separator ADT_makeSeparatorDefault(ATerm termPattern) */

ADT_Separator ADT_makeSeparatorDefault(ATerm termPattern)
{
  return (ADT_Separator)(ATerm) termPattern;
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool ADT_isEqualEntries(ADT_Entries arg0, ADT_Entries arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualEntry(ADT_Entry arg0, ADT_Entry arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualSeparators(ADT_Separators arg0, ADT_Separators arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualSeparator(ADT_Separator arg0, ADT_Separator arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  ADT_Entries accessors */

/*{{{  ATbool ADT_isValidEntries(ADT_Entries arg) */

ATbool ADT_isValidEntries(ADT_Entries arg)
{
  if (ADT_isEntriesEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntriesSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntriesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntriesEmpty(ADT_Entries arg) */

inline ATbool ADT_isEntriesEmpty(ADT_Entries arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternEntriesEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntriesSingle(ADT_Entries arg) */

inline ATbool ADT_isEntriesSingle(ADT_Entries arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntriesSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntriesMany(ADT_Entries arg) */

inline ATbool ADT_isEntriesMany(ADT_Entries arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntriesMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasEntriesHead(ADT_Entries arg) */

ATbool ADT_hasEntriesHead(ADT_Entries arg)
{
  if (ADT_isEntriesSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntriesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntriesTail(ADT_Entries arg) */

ATbool ADT_hasEntriesTail(ADT_Entries arg)
{
  if (ADT_isEntriesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_Entry ADT_getEntriesHead(ADT_Entries arg) */

ADT_Entry ADT_getEntriesHead(ADT_Entries arg)
{
  if (ADT_isEntriesSingle(arg)) {
    return (ADT_Entry)ATgetFirst((ATermList)arg);
  }
  else 
    return (ADT_Entry)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Entries ADT_getEntriesTail(ADT_Entries arg) */

ADT_Entries ADT_getEntriesTail(ADT_Entries arg)
{
  
    return (ADT_Entries)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Entries ADT_setEntriesHead(ADT_Entries arg, ADT_Entry head) */

ADT_Entries ADT_setEntriesHead(ADT_Entries arg, ADT_Entry head)
{
  if (ADT_isEntriesSingle(arg)) {
    return (ADT_Entries)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (ADT_isEntriesMany(arg)) {
    return (ADT_Entries)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Entries has no Head: %t\n", arg);
  return (ADT_Entries)NULL;
}

/*}}}  */
/*{{{  ADT_Entries ADT_setEntriesTail(ADT_Entries arg, ADT_Entries tail) */

ADT_Entries ADT_setEntriesTail(ADT_Entries arg, ADT_Entries tail)
{
  if (ADT_isEntriesMany(arg)) {
    return (ADT_Entries)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Entries has no Tail: %t\n", arg);
  return (ADT_Entries)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Entry accessors */

/*{{{  ATbool ADT_isValidEntry(ADT_Entry arg) */

ATbool ADT_isValidEntry(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntryList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntryConstructor(ADT_Entry arg) */

inline ATbool ADT_isEntryConstructor(ADT_Entry arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntryConstructor, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntryList(ADT_Entry arg) */

inline ATbool ADT_isEntryList(ADT_Entry arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntryList, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntrySeparatedList(ADT_Entry arg) */

inline ATbool ADT_isEntrySeparatedList(ADT_Entry arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntrySeparatedList, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasEntrySort(ADT_Entry arg) */

ATbool ADT_hasEntrySort(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntryList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntryAlternative(ADT_Entry arg) */

ATbool ADT_hasEntryAlternative(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntryTermPattern(ADT_Entry arg) */

ATbool ADT_hasEntryTermPattern(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntryElemSort(ADT_Entry arg) */

ATbool ADT_hasEntryElemSort(ADT_Entry arg)
{
  if (ADT_isEntryList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntrySeparators(ADT_Entry arg) */

ATbool ADT_hasEntrySeparators(ADT_Entry arg)
{
  if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATerm ADT_getEntrySort(ADT_Entry arg) */

ATerm ADT_getEntrySort(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
  }
  else if (ADT_isEntryList(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
  }
  else 
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  ATerm ADT_getEntryAlternative(ADT_Entry arg) */

ATerm ADT_getEntryAlternative(ADT_Entry arg)
{
  
    return (ATerm)ATgetArgument((ATermAppl)arg, 1);
}

/*}}}  */
/*{{{  ATerm ADT_getEntryTermPattern(ADT_Entry arg) */

ATerm ADT_getEntryTermPattern(ADT_Entry arg)
{
  
    return (ATerm)ATgetArgument((ATermAppl)arg, 2);
}

/*}}}  */
/*{{{  ATerm ADT_getEntryElemSort(ADT_Entry arg) */

ATerm ADT_getEntryElemSort(ADT_Entry arg)
{
  if (ADT_isEntryList(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 1);
  }
  else 
    return (ATerm)ATgetArgument((ATermAppl)arg, 1);
}

/*}}}  */
/*{{{  ADT_Separators ADT_getEntrySeparators(ADT_Entry arg) */

ADT_Separators ADT_getEntrySeparators(ADT_Entry arg)
{
  
    return (ADT_Separators)ATgetArgument((ATermAppl)arg, 2);
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntrySort(ADT_Entry arg, ATerm sort) */

ADT_Entry ADT_setEntrySort(ADT_Entry arg, ATerm sort)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sort), 0);
  }
  else if (ADT_isEntryList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sort), 0);
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sort), 0);
  }

  ATabort("Entry has no Sort: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntryAlternative(ADT_Entry arg, ATerm alternative) */

ADT_Entry ADT_setEntryAlternative(ADT_Entry arg, ATerm alternative)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) alternative), 1);
  }

  ATabort("Entry has no Alternative: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntryTermPattern(ADT_Entry arg, ATerm termPattern) */

ADT_Entry ADT_setEntryTermPattern(ADT_Entry arg, ATerm termPattern)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) termPattern), 2);
  }

  ATabort("Entry has no TermPattern: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntryElemSort(ADT_Entry arg, ATerm elemSort) */

ADT_Entry ADT_setEntryElemSort(ADT_Entry arg, ATerm elemSort)
{
  if (ADT_isEntryList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) elemSort), 1);
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) elemSort), 1);
  }

  ATabort("Entry has no ElemSort: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntrySeparators(ADT_Entry arg, ADT_Separators separators) */

ADT_Entry ADT_setEntrySeparators(ADT_Entry arg, ADT_Separators separators)
{
  if (ADT_isEntrySeparatedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) separators), 2);
  }

  ATabort("Entry has no Separators: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Separators accessors */

/*{{{  ATbool ADT_isValidSeparators(ADT_Separators arg) */

ATbool ADT_isValidSeparators(ADT_Separators arg)
{
  if (ADT_isSeparatorsEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT_isSeparatorsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isSeparatorsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorsEmpty(ADT_Separators arg) */

inline ATbool ADT_isSeparatorsEmpty(ADT_Separators arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternSeparatorsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorsSingle(ADT_Separators arg) */

inline ATbool ADT_isSeparatorsSingle(ADT_Separators arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternSeparatorsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorsMany(ADT_Separators arg) */

inline ATbool ADT_isSeparatorsMany(ADT_Separators arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternSeparatorsMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasSeparatorsHead(ADT_Separators arg) */

ATbool ADT_hasSeparatorsHead(ADT_Separators arg)
{
  if (ADT_isSeparatorsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isSeparatorsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasSeparatorsTail(ADT_Separators arg) */

ATbool ADT_hasSeparatorsTail(ADT_Separators arg)
{
  if (ADT_isSeparatorsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_Separator ADT_getSeparatorsHead(ADT_Separators arg) */

ADT_Separator ADT_getSeparatorsHead(ADT_Separators arg)
{
  if (ADT_isSeparatorsSingle(arg)) {
    return (ADT_Separator)ATgetFirst((ATermList)arg);
  }
  else 
    return (ADT_Separator)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Separators ADT_getSeparatorsTail(ADT_Separators arg) */

ADT_Separators ADT_getSeparatorsTail(ADT_Separators arg)
{
  
    return (ADT_Separators)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Separators ADT_setSeparatorsHead(ADT_Separators arg, ADT_Separator head) */

ADT_Separators ADT_setSeparatorsHead(ADT_Separators arg, ADT_Separator head)
{
  if (ADT_isSeparatorsSingle(arg)) {
    return (ADT_Separators)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (ADT_isSeparatorsMany(arg)) {
    return (ADT_Separators)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Separators has no Head: %t\n", arg);
  return (ADT_Separators)NULL;
}

/*}}}  */
/*{{{  ADT_Separators ADT_setSeparatorsTail(ADT_Separators arg, ADT_Separators tail) */

ADT_Separators ADT_setSeparatorsTail(ADT_Separators arg, ADT_Separators tail)
{
  if (ADT_isSeparatorsMany(arg)) {
    return (ADT_Separators)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Separators has no Tail: %t\n", arg);
  return (ADT_Separators)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Separator accessors */

/*{{{  ATbool ADT_isValidSeparator(ADT_Separator arg) */

ATbool ADT_isValidSeparator(ADT_Separator arg)
{
  if (ADT_isSeparatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorDefault(ADT_Separator arg) */

inline ATbool ADT_isSeparatorDefault(ADT_Separator arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternSeparatorDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADT_hasSeparatorTermPattern(ADT_Separator arg) */

ATbool ADT_hasSeparatorTermPattern(ADT_Separator arg)
{
  if (ADT_isSeparatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATerm ADT_getSeparatorTermPattern(ADT_Separator arg) */

ATerm ADT_getSeparatorTermPattern(ADT_Separator arg)
{
  
    return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Separator ADT_setSeparatorTermPattern(ADT_Separator arg, ATerm termPattern) */

ADT_Separator ADT_setSeparatorTermPattern(ADT_Separator arg, ATerm termPattern)
{
  if (ADT_isSeparatorDefault(arg)) {
    return (ADT_Separator)((ATerm) termPattern);
  }

  ATabort("Separator has no TermPattern: %t\n", arg);
  return (ADT_Separator)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  ADT_Entries ADT_visitEntries(ADT_Entries arg, ADT_Entry (*acceptHead)(ADT_Entry)) */

ADT_Entries ADT_visitEntries(ADT_Entries arg, ADT_Entry (*acceptHead)(ADT_Entry))
{
  if (ADT_isEntriesEmpty(arg)) {
    return ADT_makeEntriesEmpty();
  }
  if (ADT_isEntriesSingle(arg)) {
    return ADT_makeEntriesSingle(
        acceptHead ? acceptHead(ADT_getEntriesHead(arg)) : ADT_getEntriesHead(arg));
  }
  if (ADT_isEntriesMany(arg)) {
    return ADT_makeEntriesMany(
        acceptHead ? acceptHead(ADT_getEntriesHead(arg)) : ADT_getEntriesHead(arg),
        ADT_visitEntries(ADT_getEntriesTail(arg), acceptHead));
  }
  ATabort("not a Entries: %t\n", arg);
  return (ADT_Entries)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_visitEntry(ADT_Entry arg, ATerm (*acceptSort)(ATerm), ATerm (*acceptAlternative)(ATerm), ATerm (*acceptTermPattern)(ATerm), ATerm (*acceptElemSort)(ATerm), ADT_Separators (*acceptSeparators)(ADT_Separators)) */

ADT_Entry ADT_visitEntry(ADT_Entry arg, ATerm (*acceptSort)(ATerm), ATerm (*acceptAlternative)(ATerm), ATerm (*acceptTermPattern)(ATerm), ATerm (*acceptElemSort)(ATerm), ADT_Separators (*acceptSeparators)(ADT_Separators))
{
  if (ADT_isEntryConstructor(arg)) {
    return ADT_makeEntryConstructor(
        acceptSort ? acceptSort(ADT_getEntrySort(arg)) : ADT_getEntrySort(arg),
        acceptAlternative ? acceptAlternative(ADT_getEntryAlternative(arg)) : ADT_getEntryAlternative(arg),
        acceptTermPattern ? acceptTermPattern(ADT_getEntryTermPattern(arg)) : ADT_getEntryTermPattern(arg));
  }
  if (ADT_isEntryList(arg)) {
    return ADT_makeEntryList(
        acceptSort ? acceptSort(ADT_getEntrySort(arg)) : ADT_getEntrySort(arg),
        acceptElemSort ? acceptElemSort(ADT_getEntryElemSort(arg)) : ADT_getEntryElemSort(arg));
  }
  if (ADT_isEntrySeparatedList(arg)) {
    return ADT_makeEntrySeparatedList(
        acceptSort ? acceptSort(ADT_getEntrySort(arg)) : ADT_getEntrySort(arg),
        acceptElemSort ? acceptElemSort(ADT_getEntryElemSort(arg)) : ADT_getEntryElemSort(arg),
        acceptSeparators ? acceptSeparators(ADT_getEntrySeparators(arg)) : ADT_getEntrySeparators(arg));
  }
  ATabort("not a Entry: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Separators ADT_visitSeparators(ADT_Separators arg, ADT_Separator (*acceptHead)(ADT_Separator)) */

ADT_Separators ADT_visitSeparators(ADT_Separators arg, ADT_Separator (*acceptHead)(ADT_Separator))
{
  if (ADT_isSeparatorsEmpty(arg)) {
    return ADT_makeSeparatorsEmpty();
  }
  if (ADT_isSeparatorsSingle(arg)) {
    return ADT_makeSeparatorsSingle(
        acceptHead ? acceptHead(ADT_getSeparatorsHead(arg)) : ADT_getSeparatorsHead(arg));
  }
  if (ADT_isSeparatorsMany(arg)) {
    return ADT_makeSeparatorsMany(
        acceptHead ? acceptHead(ADT_getSeparatorsHead(arg)) : ADT_getSeparatorsHead(arg),
        ADT_visitSeparators(ADT_getSeparatorsTail(arg), acceptHead));
  }
  ATabort("not a Separators: %t\n", arg);
  return (ADT_Separators)NULL;
}

/*}}}  */
/*{{{  ADT_Separator ADT_visitSeparator(ADT_Separator arg, ATerm (*acceptTermPattern)(ATerm)) */

ADT_Separator ADT_visitSeparator(ADT_Separator arg, ATerm (*acceptTermPattern)(ATerm))
{
  if (ADT_isSeparatorDefault(arg)) {
    return ADT_makeSeparatorDefault(
        acceptTermPattern ? acceptTermPattern(ADT_getSeparatorTermPattern(arg)) : ADT_getSeparatorTermPattern(arg));
  }
  ATabort("not a Separator: %t\n", arg);
  return (ADT_Separator)NULL;
}

/*}}}  */

/*}}}  */
