#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "ADT.h"

/*{{{  conversion functions */

ATerm ADT_stringToChars(const char *str)
{
  int len = strlen(str);
  int i;
  ATermList result = ATempty;

  for (i = len - 1; i >= 0; i--) {
    result = ATinsert(result, (ATerm) ATmakeInt(str[i]));
  }

  return (ATerm) result;
}

ATerm ADT_byteToChar(char ch)
{
    return (ATerm) ATmakeInt(ch);
}

char *ADT_charsToString(ATerm arg)
{
  ATermList list = (ATermList) arg;
  int len = ATgetLength(list);
  int i;
  char *str;

  str = (char *) malloc(len+1);
  if (str == NULL) {
      return NULL;
  }

  for (i = 0; !ATisEmpty(list); list = ATgetNext(list), i++) {
    str[i] = (char) ATgetInt((ATermInt) ATgetFirst(list));
  }
  str[i] = '\0';

  return str;
}

char ADT_charToByte(ATerm arg)
{
    return (char) ATgetInt((ATermInt) arg);
}


/*}}}  */

/*{{{  typedefs */

typedef struct ATerm _ADT_Entries;
typedef struct ATerm _ADT_Entry;
typedef struct ATerm _ADT_Separators;
typedef struct ATerm _ADT_Separator;
typedef struct ATerm _ADT_Modules;
typedef struct ATerm _ADT_Module;
typedef struct ATerm _ADT_Imports;
typedef struct ATerm _ADT_Type;
typedef struct ATerm _ADT_Sorts;
typedef struct ATerm _ADT_ModuleName;

/*}}}  */

/*{{{  void ADT_initADTApi(void) */

void ADT_initADTApi(void)
{
  init_ADT_dict();
}

/*}}}  */

/*{{{  (un)protect functions */

void ADT_protectEntries(ADT_Entries *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectEntries(ADT_Entries *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectEntry(ADT_Entry *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectEntry(ADT_Entry *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectSeparators(ADT_Separators *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectSeparators(ADT_Separators *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectSeparator(ADT_Separator *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectSeparator(ADT_Separator *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectModules(ADT_Modules *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectModules(ADT_Modules *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectModule(ADT_Module *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectModule(ADT_Module *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectImports(ADT_Imports *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectImports(ADT_Imports *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectType(ADT_Type *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectType(ADT_Type *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectSorts(ADT_Sorts *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectSorts(ADT_Sorts *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}

void ADT_protectModuleName(ADT_ModuleName *arg)
{
  ATprotect((ATerm*)((void*) arg));
}

void ADT_unprotectModuleName(ADT_ModuleName *arg)
{
  ATunprotect((ATerm*)((void*) arg));
}


/*}}}  */
/*{{{  term conversion functions */

/*{{{  ADT_Entries ADT_EntriesFromTerm(ATerm t) */

ADT_Entries ADT_EntriesFromTerm(ATerm t)
{
  return (ADT_Entries)t;
}

/*}}}  */
/*{{{  ATerm ADT_EntriesToTerm(ADT_Entries arg) */

ATerm ADT_EntriesToTerm(ADT_Entries arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Entry ADT_EntryFromTerm(ATerm t) */

ADT_Entry ADT_EntryFromTerm(ATerm t)
{
  return (ADT_Entry)t;
}

/*}}}  */
/*{{{  ATerm ADT_EntryToTerm(ADT_Entry arg) */

ATerm ADT_EntryToTerm(ADT_Entry arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Separators ADT_SeparatorsFromTerm(ATerm t) */

ADT_Separators ADT_SeparatorsFromTerm(ATerm t)
{
  return (ADT_Separators)t;
}

/*}}}  */
/*{{{  ATerm ADT_SeparatorsToTerm(ADT_Separators arg) */

ATerm ADT_SeparatorsToTerm(ADT_Separators arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Separator ADT_SeparatorFromTerm(ATerm t) */

ADT_Separator ADT_SeparatorFromTerm(ATerm t)
{
  return (ADT_Separator)t;
}

/*}}}  */
/*{{{  ATerm ADT_SeparatorToTerm(ADT_Separator arg) */

ATerm ADT_SeparatorToTerm(ADT_Separator arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Modules ADT_ModulesFromTerm(ATerm t) */

ADT_Modules ADT_ModulesFromTerm(ATerm t)
{
  return (ADT_Modules)t;
}

/*}}}  */
/*{{{  ATerm ADT_ModulesToTerm(ADT_Modules arg) */

ATerm ADT_ModulesToTerm(ADT_Modules arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Module ADT_ModuleFromTerm(ATerm t) */

ADT_Module ADT_ModuleFromTerm(ATerm t)
{
  return (ADT_Module)t;
}

/*}}}  */
/*{{{  ATerm ADT_ModuleToTerm(ADT_Module arg) */

ATerm ADT_ModuleToTerm(ADT_Module arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Imports ADT_ImportsFromTerm(ATerm t) */

ADT_Imports ADT_ImportsFromTerm(ATerm t)
{
  return (ADT_Imports)t;
}

/*}}}  */
/*{{{  ATerm ADT_ImportsToTerm(ADT_Imports arg) */

ATerm ADT_ImportsToTerm(ADT_Imports arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Type ADT_TypeFromTerm(ATerm t) */

ADT_Type ADT_TypeFromTerm(ATerm t)
{
  return (ADT_Type)t;
}

/*}}}  */
/*{{{  ATerm ADT_TypeToTerm(ADT_Type arg) */

ATerm ADT_TypeToTerm(ADT_Type arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Sorts ADT_SortsFromTerm(ATerm t) */

ADT_Sorts ADT_SortsFromTerm(ATerm t)
{
  return (ADT_Sorts)t;
}

/*}}}  */
/*{{{  ATerm ADT_SortsToTerm(ADT_Sorts arg) */

ATerm ADT_SortsToTerm(ADT_Sorts arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_ModuleName ADT_ModuleNameFromTerm(ATerm t) */

ADT_ModuleName ADT_ModuleNameFromTerm(ATerm t)
{
  return (ADT_ModuleName)t;
}

/*}}}  */
/*{{{  ATerm ADT_ModuleNameToTerm(ADT_ModuleName arg) */

ATerm ADT_ModuleNameToTerm(ADT_ModuleName arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  list functions */

int ADT_getEntriesLength (ADT_Entries arg) {
  return ATgetLength((ATermList) arg);
}
ADT_Entries ADT_reverseEntries(ADT_Entries arg) {
  return (ADT_Entries) ATreverse((ATermList) arg);
}
ADT_Entries ADT_appendEntries(ADT_Entries arg, ADT_Entry elem) {
  return (ADT_Entries) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
ADT_Entries ADT_concatEntries(ADT_Entries arg0, ADT_Entries arg1) {
  return (ADT_Entries) ATconcat((ATermList) arg0, (ATermList) arg1);
}
ADT_Entries ADT_sliceEntries(ADT_Entries arg, int start, int end) {
  return (ADT_Entries) ATgetSlice((ATermList) arg, start, end);
}
ADT_Entry ADT_getEntriesEntryAt(ADT_Entries arg, int index) {
 return (ADT_Entry)ATelementAt((ATermList) arg,index);
}
ADT_Entries ADT_replaceEntriesEntryAt(ADT_Entries arg, ADT_Entry elem, int index) {
 return (ADT_Entries) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
ADT_Entries ADT_makeEntries2(ADT_Entry elem1, ADT_Entry elem2) {
  return (ADT_Entries) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
ADT_Entries ADT_makeEntries3(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3) {
  return (ADT_Entries) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
ADT_Entries ADT_makeEntries4(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4) {
  return (ADT_Entries) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
ADT_Entries ADT_makeEntries5(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4, ADT_Entry elem5) {
  return (ADT_Entries) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
ADT_Entries ADT_makeEntries6(ADT_Entry elem1, ADT_Entry elem2, ADT_Entry elem3, ADT_Entry elem4, ADT_Entry elem5, ADT_Entry elem6) {
  return (ADT_Entries) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int ADT_getSeparatorsLength (ADT_Separators arg) {
  return ATgetLength((ATermList) arg);
}
ADT_Separators ADT_reverseSeparators(ADT_Separators arg) {
  return (ADT_Separators) ATreverse((ATermList) arg);
}
ADT_Separators ADT_appendSeparators(ADT_Separators arg, ADT_Separator elem) {
  return (ADT_Separators) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
ADT_Separators ADT_concatSeparators(ADT_Separators arg0, ADT_Separators arg1) {
  return (ADT_Separators) ATconcat((ATermList) arg0, (ATermList) arg1);
}
ADT_Separators ADT_sliceSeparators(ADT_Separators arg, int start, int end) {
  return (ADT_Separators) ATgetSlice((ATermList) arg, start, end);
}
ADT_Separator ADT_getSeparatorsSeparatorAt(ADT_Separators arg, int index) {
 return (ADT_Separator)ATelementAt((ATermList) arg,index);
}
ADT_Separators ADT_replaceSeparatorsSeparatorAt(ADT_Separators arg, ADT_Separator elem, int index) {
 return (ADT_Separators) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
ADT_Separators ADT_makeSeparators2(ADT_Separator elem1, ADT_Separator elem2) {
  return (ADT_Separators) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
ADT_Separators ADT_makeSeparators3(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3) {
  return (ADT_Separators) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
ADT_Separators ADT_makeSeparators4(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4) {
  return (ADT_Separators) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
ADT_Separators ADT_makeSeparators5(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4, ADT_Separator elem5) {
  return (ADT_Separators) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
ADT_Separators ADT_makeSeparators6(ADT_Separator elem1, ADT_Separator elem2, ADT_Separator elem3, ADT_Separator elem4, ADT_Separator elem5, ADT_Separator elem6) {
  return (ADT_Separators) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int ADT_getModulesLength (ADT_Modules arg) {
  return ATgetLength((ATermList) arg);
}
ADT_Modules ADT_reverseModules(ADT_Modules arg) {
  return (ADT_Modules) ATreverse((ATermList) arg);
}
ADT_Modules ADT_appendModules(ADT_Modules arg, ADT_Module elem) {
  return (ADT_Modules) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
ADT_Modules ADT_concatModules(ADT_Modules arg0, ADT_Modules arg1) {
  return (ADT_Modules) ATconcat((ATermList) arg0, (ATermList) arg1);
}
ADT_Modules ADT_sliceModules(ADT_Modules arg, int start, int end) {
  return (ADT_Modules) ATgetSlice((ATermList) arg, start, end);
}
ADT_Module ADT_getModulesModuleAt(ADT_Modules arg, int index) {
 return (ADT_Module)ATelementAt((ATermList) arg,index);
}
ADT_Modules ADT_replaceModulesModuleAt(ADT_Modules arg, ADT_Module elem, int index) {
 return (ADT_Modules) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
ADT_Modules ADT_makeModules2(ADT_Module elem1, ADT_Module elem2) {
  return (ADT_Modules) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
ADT_Modules ADT_makeModules3(ADT_Module elem1, ADT_Module elem2, ADT_Module elem3) {
  return (ADT_Modules) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
ADT_Modules ADT_makeModules4(ADT_Module elem1, ADT_Module elem2, ADT_Module elem3, ADT_Module elem4) {
  return (ADT_Modules) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
ADT_Modules ADT_makeModules5(ADT_Module elem1, ADT_Module elem2, ADT_Module elem3, ADT_Module elem4, ADT_Module elem5) {
  return (ADT_Modules) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
ADT_Modules ADT_makeModules6(ADT_Module elem1, ADT_Module elem2, ADT_Module elem3, ADT_Module elem4, ADT_Module elem5, ADT_Module elem6) {
  return (ADT_Modules) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int ADT_getImportsLength (ADT_Imports arg) {
  return ATgetLength((ATermList) arg);
}
ADT_Imports ADT_reverseImports(ADT_Imports arg) {
  return (ADT_Imports) ATreverse((ATermList) arg);
}
ADT_Imports ADT_appendImports(ADT_Imports arg, ADT_ModuleName elem) {
  return (ADT_Imports) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
ADT_Imports ADT_concatImports(ADT_Imports arg0, ADT_Imports arg1) {
  return (ADT_Imports) ATconcat((ATermList) arg0, (ATermList) arg1);
}
ADT_Imports ADT_sliceImports(ADT_Imports arg, int start, int end) {
  return (ADT_Imports) ATgetSlice((ATermList) arg, start, end);
}
ADT_ModuleName ADT_getImportsModuleNameAt(ADT_Imports arg, int index) {
 return (ADT_ModuleName)ATelementAt((ATermList) arg,index);
}
ADT_Imports ADT_replaceImportsModuleNameAt(ADT_Imports arg, ADT_ModuleName elem, int index) {
 return (ADT_Imports) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
ADT_Imports ADT_makeImports2(ADT_ModuleName elem1, ADT_ModuleName elem2) {
  return (ADT_Imports) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
ADT_Imports ADT_makeImports3(ADT_ModuleName elem1, ADT_ModuleName elem2, ADT_ModuleName elem3) {
  return (ADT_Imports) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
ADT_Imports ADT_makeImports4(ADT_ModuleName elem1, ADT_ModuleName elem2, ADT_ModuleName elem3, ADT_ModuleName elem4) {
  return (ADT_Imports) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
ADT_Imports ADT_makeImports5(ADT_ModuleName elem1, ADT_ModuleName elem2, ADT_ModuleName elem3, ADT_ModuleName elem4, ADT_ModuleName elem5) {
  return (ADT_Imports) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
ADT_Imports ADT_makeImports6(ADT_ModuleName elem1, ADT_ModuleName elem2, ADT_ModuleName elem3, ADT_ModuleName elem4, ADT_ModuleName elem5, ADT_ModuleName elem6) {
  return (ADT_Imports) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}
int ADT_getSortsLength (ADT_Sorts arg) {
  return ATgetLength((ATermList) arg);
}
ADT_Sorts ADT_reverseSorts(ADT_Sorts arg) {
  return (ADT_Sorts) ATreverse((ATermList) arg);
}
ADT_Sorts ADT_appendSorts(ADT_Sorts arg, ADT_Type elem) {
  return (ADT_Sorts) ATappend((ATermList) arg, (ATerm) ((ATerm) elem));
}
ADT_Sorts ADT_concatSorts(ADT_Sorts arg0, ADT_Sorts arg1) {
  return (ADT_Sorts) ATconcat((ATermList) arg0, (ATermList) arg1);
}
ADT_Sorts ADT_sliceSorts(ADT_Sorts arg, int start, int end) {
  return (ADT_Sorts) ATgetSlice((ATermList) arg, start, end);
}
ADT_Type ADT_getSortsTypeAt(ADT_Sorts arg, int index) {
 return (ADT_Type)ATelementAt((ATermList) arg,index);
}
ADT_Sorts ADT_replaceSortsTypeAt(ADT_Sorts arg, ADT_Type elem, int index) {
 return (ADT_Sorts) ATreplace((ATermList) arg, (ATerm) ((ATerm) elem), index);
}
ADT_Sorts ADT_makeSorts2(ADT_Type elem1, ADT_Type elem2) {
  return (ADT_Sorts) ATmakeList2((ATerm) ((ATerm) elem2), (ATerm) ((ATerm) elem2));
}
ADT_Sorts ADT_makeSorts3(ADT_Type elem1, ADT_Type elem2, ADT_Type elem3) {
  return (ADT_Sorts) ATmakeList3((ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3), (ATerm) ((ATerm) elem3));
}
ADT_Sorts ADT_makeSorts4(ADT_Type elem1, ADT_Type elem2, ADT_Type elem3, ADT_Type elem4) {
  return (ADT_Sorts) ATmakeList4((ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4), (ATerm) ((ATerm) elem4));
}
ADT_Sorts ADT_makeSorts5(ADT_Type elem1, ADT_Type elem2, ADT_Type elem3, ADT_Type elem4, ADT_Type elem5) {
  return (ADT_Sorts) ATmakeList5((ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5), (ATerm) ((ATerm) elem5));
}
ADT_Sorts ADT_makeSorts6(ADT_Type elem1, ADT_Type elem2, ADT_Type elem3, ADT_Type elem4, ADT_Type elem5, ADT_Type elem6) {
  return (ADT_Sorts) ATmakeList6((ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6), (ATerm) ((ATerm) elem6));
}

/*}}}  */
/*{{{  constructors */

/*{{{  ADT_Entries ADT_makeEntriesEmpty(void) */

ADT_Entries ADT_makeEntriesEmpty(void)
{
  return (ADT_Entries)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT_Entries ADT_makeEntriesSingle(ADT_Entry head) */

ADT_Entries ADT_makeEntriesSingle(ADT_Entry head)
{
  return (ADT_Entries)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  ADT_Entries ADT_makeEntriesMany(ADT_Entry head, ADT_Entries tail) */

ADT_Entries ADT_makeEntriesMany(ADT_Entry head, ADT_Entries tail)
{
  return (ADT_Entries)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT_Entry ADT_makeEntryConstructor(ATerm sort, ATerm alternative, ATerm termPattern) */

ADT_Entry ADT_makeEntryConstructor(ATerm sort, ATerm alternative, ATerm termPattern)
{
  return (ADT_Entry)(ATerm)ATmakeAppl3(ADT_afun0, (ATerm) sort, (ATerm) alternative, (ATerm) termPattern);
}

/*}}}  */
/*{{{  ADT_Entry ADT_makeEntryList(ATerm sort, ATerm elemSort) */

ADT_Entry ADT_makeEntryList(ATerm sort, ATerm elemSort)
{
  return (ADT_Entry)(ATerm)ATmakeAppl2(ADT_afun1, (ATerm) sort, (ATerm) elemSort);
}

/*}}}  */
/*{{{  ADT_Entry ADT_makeEntryNamedList(ATerm opname, ATerm sort, ATerm elemSort) */

ADT_Entry ADT_makeEntryNamedList(ATerm opname, ATerm sort, ATerm elemSort)
{
  return (ADT_Entry)(ATerm)ATmakeAppl3(ADT_afun2, (ATerm) opname, (ATerm) sort, (ATerm) elemSort);
}

/*}}}  */
/*{{{  ADT_Entry ADT_makeEntrySeparatedList(ATerm sort, ATerm elemSort, ADT_Separators separators) */

ADT_Entry ADT_makeEntrySeparatedList(ATerm sort, ATerm elemSort, ADT_Separators separators)
{
  return (ADT_Entry)(ATerm)ATmakeAppl3(ADT_afun3, (ATerm) sort, (ATerm) elemSort, (ATerm) separators);
}

/*}}}  */
/*{{{  ADT_Separators ADT_makeSeparatorsEmpty(void) */

ADT_Separators ADT_makeSeparatorsEmpty(void)
{
  return (ADT_Separators)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT_Separators ADT_makeSeparatorsSingle(ADT_Separator head) */

ADT_Separators ADT_makeSeparatorsSingle(ADT_Separator head)
{
  return (ADT_Separators)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  ADT_Separators ADT_makeSeparatorsMany(ADT_Separator head, ADT_Separators tail) */

ADT_Separators ADT_makeSeparatorsMany(ADT_Separator head, ADT_Separators tail)
{
  return (ADT_Separators)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT_Separator ADT_makeSeparatorDefault(ATerm termPattern) */

ADT_Separator ADT_makeSeparatorDefault(ATerm termPattern)
{
  return (ADT_Separator)(ATerm) termPattern;
}

/*}}}  */
/*{{{  ADT_Modules ADT_makeModulesEmpty(void) */

ADT_Modules ADT_makeModulesEmpty(void)
{
  return (ADT_Modules)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT_Modules ADT_makeModulesSingle(ADT_Module head) */

ADT_Modules ADT_makeModulesSingle(ADT_Module head)
{
  return (ADT_Modules)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  ADT_Modules ADT_makeModulesMany(ADT_Module head, ADT_Modules tail) */

ADT_Modules ADT_makeModulesMany(ADT_Module head, ADT_Modules tail)
{
  return (ADT_Modules)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT_Module ADT_makeModuleModulentry(ADT_ModuleName modulename, ADT_Imports imports, ADT_Sorts sorts, ADT_Entries entries) */

ADT_Module ADT_makeModuleModulentry(ADT_ModuleName modulename, ADT_Imports imports, ADT_Sorts sorts, ADT_Entries entries)
{
  return (ADT_Module)(ATerm)ATmakeAppl4(ADT_afun4, (ATerm) modulename, (ATerm) imports, (ATerm) sorts, (ATerm) entries);
}

/*}}}  */
/*{{{  ADT_Imports ADT_makeImportsEmpty(void) */

ADT_Imports ADT_makeImportsEmpty(void)
{
  return (ADT_Imports)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT_Imports ADT_makeImportsSingle(ADT_ModuleName head) */

ADT_Imports ADT_makeImportsSingle(ADT_ModuleName head)
{
  return (ADT_Imports)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  ADT_Imports ADT_makeImportsMany(ADT_ModuleName head, ADT_Imports tail) */

ADT_Imports ADT_makeImportsMany(ADT_ModuleName head, ADT_Imports tail)
{
  return (ADT_Imports)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT_Type ADT_makeTypeType(const char* name) */

ADT_Type ADT_makeTypeType(const char* name)
{
  return (ADT_Type)(ATerm)ATmakeAppl1(ADT_afun5, (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(name, 0, ATtrue)));
}

/*}}}  */
/*{{{  ADT_Sorts ADT_makeSortsEmpty(void) */

ADT_Sorts ADT_makeSortsEmpty(void)
{
  return (ADT_Sorts)(ATerm)ATempty;
}

/*}}}  */
/*{{{  ADT_Sorts ADT_makeSortsSingle(ADT_Type head) */

ADT_Sorts ADT_makeSortsSingle(ADT_Type head)
{
  return (ADT_Sorts)(ATerm)ATmakeList1((ATerm) head);
}

/*}}}  */
/*{{{  ADT_Sorts ADT_makeSortsMany(ADT_Type head, ADT_Sorts tail) */

ADT_Sorts ADT_makeSortsMany(ADT_Type head, ADT_Sorts tail)
{
  return (ADT_Sorts)(ATerm)ATinsert((ATermList)tail, (ATerm) head);
}

/*}}}  */
/*{{{  ADT_ModuleName ADT_makeModuleNameName(const char* name) */

ADT_ModuleName ADT_makeModuleNameName(const char* name)
{
  return (ADT_ModuleName)(ATerm)ATmakeAppl1(ADT_afun6, (ATerm) (ATerm) ATmakeAppl(ATmakeAFun(name, 0, ATtrue)));
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool ADT_isEqualEntries(ADT_Entries arg0, ADT_Entries arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualEntry(ADT_Entry arg0, ADT_Entry arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualSeparators(ADT_Separators arg0, ADT_Separators arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualSeparator(ADT_Separator arg0, ADT_Separator arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualModules(ADT_Modules arg0, ADT_Modules arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualModule(ADT_Module arg0, ADT_Module arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualImports(ADT_Imports arg0, ADT_Imports arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualType(ADT_Type arg0, ADT_Type arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualSorts(ADT_Sorts arg0, ADT_Sorts arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADT_isEqualModuleName(ADT_ModuleName arg0, ADT_ModuleName arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  ADT_Entries accessors */

/*{{{  ATbool ADT_isValidEntries(ADT_Entries arg) */

ATbool ADT_isValidEntries(ADT_Entries arg)
{
  if (ADT_isEntriesEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntriesSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntriesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntriesEmpty(ADT_Entries arg) */

inline ATbool ADT_isEntriesEmpty(ADT_Entries arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternEntriesEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntriesSingle(ADT_Entries arg) */

inline ATbool ADT_isEntriesSingle(ADT_Entries arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntriesSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntriesMany(ADT_Entries arg) */

inline ATbool ADT_isEntriesMany(ADT_Entries arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntriesMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasEntriesHead(ADT_Entries arg) */

ATbool ADT_hasEntriesHead(ADT_Entries arg)
{
  if (ADT_isEntriesSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntriesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntriesTail(ADT_Entries arg) */

ATbool ADT_hasEntriesTail(ADT_Entries arg)
{
  if (ADT_isEntriesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_Entry ADT_getEntriesHead(ADT_Entries arg) */

ADT_Entry ADT_getEntriesHead(ADT_Entries arg)
{
  if (ADT_isEntriesSingle(arg)) {
    return (ADT_Entry)ATgetFirst((ATermList)arg);
  }
  else 
    return (ADT_Entry)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Entries ADT_getEntriesTail(ADT_Entries arg) */

ADT_Entries ADT_getEntriesTail(ADT_Entries arg)
{
  
    return (ADT_Entries)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Entries ADT_setEntriesHead(ADT_Entries arg, ADT_Entry head) */

ADT_Entries ADT_setEntriesHead(ADT_Entries arg, ADT_Entry head)
{
  if (ADT_isEntriesSingle(arg)) {
    return (ADT_Entries)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (ADT_isEntriesMany(arg)) {
    return (ADT_Entries)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Entries has no Head: %t\n", arg);
  return (ADT_Entries)NULL;
}

/*}}}  */
/*{{{  ADT_Entries ADT_setEntriesTail(ADT_Entries arg, ADT_Entries tail) */

ADT_Entries ADT_setEntriesTail(ADT_Entries arg, ADT_Entries tail)
{
  if (ADT_isEntriesMany(arg)) {
    return (ADT_Entries)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Entries has no Tail: %t\n", arg);
  return (ADT_Entries)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Entry accessors */

/*{{{  ATbool ADT_isValidEntry(ADT_Entry arg) */

ATbool ADT_isValidEntry(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntryList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntryNamedList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntryConstructor(ADT_Entry arg) */

inline ATbool ADT_isEntryConstructor(ADT_Entry arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntryConstructor, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntryList(ADT_Entry arg) */

inline ATbool ADT_isEntryList(ADT_Entry arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntryList, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntryNamedList(ADT_Entry arg) */

inline ATbool ADT_isEntryNamedList(ADT_Entry arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntryNamedList, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isEntrySeparatedList(ADT_Entry arg) */

inline ATbool ADT_isEntrySeparatedList(ADT_Entry arg)
{
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternEntrySeparatedList, NULL, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasEntrySort(ADT_Entry arg) */

ATbool ADT_hasEntrySort(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntryList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntryNamedList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntryAlternative(ADT_Entry arg) */

ATbool ADT_hasEntryAlternative(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntryTermPattern(ADT_Entry arg) */

ATbool ADT_hasEntryTermPattern(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntryElemSort(ADT_Entry arg) */

ATbool ADT_hasEntryElemSort(ADT_Entry arg)
{
  if (ADT_isEntryList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntryNamedList(arg)) {
    return ATtrue;
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntryOpname(ADT_Entry arg) */

ATbool ADT_hasEntryOpname(ADT_Entry arg)
{
  if (ADT_isEntryNamedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasEntrySeparators(ADT_Entry arg) */

ATbool ADT_hasEntrySeparators(ADT_Entry arg)
{
  if (ADT_isEntrySeparatedList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATerm ADT_getEntrySort(ADT_Entry arg) */

ATerm ADT_getEntrySort(ADT_Entry arg)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
  }
  else if (ADT_isEntryList(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
  }
  else if (ADT_isEntryNamedList(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 1);
  }
  else 
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  ATerm ADT_getEntryAlternative(ADT_Entry arg) */

ATerm ADT_getEntryAlternative(ADT_Entry arg)
{
  
    return (ATerm)ATgetArgument((ATermAppl)arg, 1);
}

/*}}}  */
/*{{{  ATerm ADT_getEntryTermPattern(ADT_Entry arg) */

ATerm ADT_getEntryTermPattern(ADT_Entry arg)
{
  
    return (ATerm)ATgetArgument((ATermAppl)arg, 2);
}

/*}}}  */
/*{{{  ATerm ADT_getEntryElemSort(ADT_Entry arg) */

ATerm ADT_getEntryElemSort(ADT_Entry arg)
{
  if (ADT_isEntryList(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 1);
  }
  else if (ADT_isEntryNamedList(arg)) {
    return (ATerm)ATgetArgument((ATermAppl)arg, 2);
  }
  else 
    return (ATerm)ATgetArgument((ATermAppl)arg, 1);
}

/*}}}  */
/*{{{  ATerm ADT_getEntryOpname(ADT_Entry arg) */

ATerm ADT_getEntryOpname(ADT_Entry arg)
{
  
    return (ATerm)ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  ADT_Separators ADT_getEntrySeparators(ADT_Entry arg) */

ADT_Separators ADT_getEntrySeparators(ADT_Entry arg)
{
  
    return (ADT_Separators)ATgetArgument((ATermAppl)arg, 2);
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntrySort(ADT_Entry arg, ATerm sort) */

ADT_Entry ADT_setEntrySort(ADT_Entry arg, ATerm sort)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sort), 0);
  }
  else if (ADT_isEntryList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sort), 0);
  }
  else if (ADT_isEntryNamedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sort), 1);
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sort), 0);
  }

  ATabort("Entry has no Sort: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntryAlternative(ADT_Entry arg, ATerm alternative) */

ADT_Entry ADT_setEntryAlternative(ADT_Entry arg, ATerm alternative)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) alternative), 1);
  }

  ATabort("Entry has no Alternative: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntryTermPattern(ADT_Entry arg, ATerm termPattern) */

ADT_Entry ADT_setEntryTermPattern(ADT_Entry arg, ATerm termPattern)
{
  if (ADT_isEntryConstructor(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) termPattern), 2);
  }

  ATabort("Entry has no TermPattern: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntryElemSort(ADT_Entry arg, ATerm elemSort) */

ADT_Entry ADT_setEntryElemSort(ADT_Entry arg, ATerm elemSort)
{
  if (ADT_isEntryList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) elemSort), 1);
  }
  else if (ADT_isEntryNamedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) elemSort), 2);
  }
  else if (ADT_isEntrySeparatedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) elemSort), 1);
  }

  ATabort("Entry has no ElemSort: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntryOpname(ADT_Entry arg, ATerm opname) */

ADT_Entry ADT_setEntryOpname(ADT_Entry arg, ATerm opname)
{
  if (ADT_isEntryNamedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) opname), 0);
  }

  ATabort("Entry has no Opname: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_setEntrySeparators(ADT_Entry arg, ADT_Separators separators) */

ADT_Entry ADT_setEntrySeparators(ADT_Entry arg, ADT_Separators separators)
{
  if (ADT_isEntrySeparatedList(arg)) {
    return (ADT_Entry)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) separators), 2);
  }

  ATabort("Entry has no Separators: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Separators accessors */

/*{{{  ATbool ADT_isValidSeparators(ADT_Separators arg) */

ATbool ADT_isValidSeparators(ADT_Separators arg)
{
  if (ADT_isSeparatorsEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT_isSeparatorsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isSeparatorsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorsEmpty(ADT_Separators arg) */

inline ATbool ADT_isSeparatorsEmpty(ADT_Separators arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternSeparatorsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorsSingle(ADT_Separators arg) */

inline ATbool ADT_isSeparatorsSingle(ADT_Separators arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternSeparatorsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorsMany(ADT_Separators arg) */

inline ATbool ADT_isSeparatorsMany(ADT_Separators arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternSeparatorsMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasSeparatorsHead(ADT_Separators arg) */

ATbool ADT_hasSeparatorsHead(ADT_Separators arg)
{
  if (ADT_isSeparatorsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isSeparatorsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasSeparatorsTail(ADT_Separators arg) */

ATbool ADT_hasSeparatorsTail(ADT_Separators arg)
{
  if (ADT_isSeparatorsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_Separator ADT_getSeparatorsHead(ADT_Separators arg) */

ADT_Separator ADT_getSeparatorsHead(ADT_Separators arg)
{
  if (ADT_isSeparatorsSingle(arg)) {
    return (ADT_Separator)ATgetFirst((ATermList)arg);
  }
  else 
    return (ADT_Separator)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Separators ADT_getSeparatorsTail(ADT_Separators arg) */

ADT_Separators ADT_getSeparatorsTail(ADT_Separators arg)
{
  
    return (ADT_Separators)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Separators ADT_setSeparatorsHead(ADT_Separators arg, ADT_Separator head) */

ADT_Separators ADT_setSeparatorsHead(ADT_Separators arg, ADT_Separator head)
{
  if (ADT_isSeparatorsSingle(arg)) {
    return (ADT_Separators)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (ADT_isSeparatorsMany(arg)) {
    return (ADT_Separators)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Separators has no Head: %t\n", arg);
  return (ADT_Separators)NULL;
}

/*}}}  */
/*{{{  ADT_Separators ADT_setSeparatorsTail(ADT_Separators arg, ADT_Separators tail) */

ADT_Separators ADT_setSeparatorsTail(ADT_Separators arg, ADT_Separators tail)
{
  if (ADT_isSeparatorsMany(arg)) {
    return (ADT_Separators)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Separators has no Tail: %t\n", arg);
  return (ADT_Separators)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Separator accessors */

/*{{{  ATbool ADT_isValidSeparator(ADT_Separator arg) */

ATbool ADT_isValidSeparator(ADT_Separator arg)
{
  if (ADT_isSeparatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSeparatorDefault(ADT_Separator arg) */

inline ATbool ADT_isSeparatorDefault(ADT_Separator arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternSeparatorDefault, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADT_hasSeparatorTermPattern(ADT_Separator arg) */

ATbool ADT_hasSeparatorTermPattern(ADT_Separator arg)
{
  if (ADT_isSeparatorDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATerm ADT_getSeparatorTermPattern(ADT_Separator arg) */

ATerm ADT_getSeparatorTermPattern(ADT_Separator arg)
{
  
    return (ATerm)arg;
}

/*}}}  */
/*{{{  ADT_Separator ADT_setSeparatorTermPattern(ADT_Separator arg, ATerm termPattern) */

ADT_Separator ADT_setSeparatorTermPattern(ADT_Separator arg, ATerm termPattern)
{
  if (ADT_isSeparatorDefault(arg)) {
    return (ADT_Separator)((ATerm) termPattern);
  }

  ATabort("Separator has no TermPattern: %t\n", arg);
  return (ADT_Separator)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Modules accessors */

/*{{{  ATbool ADT_isValidModules(ADT_Modules arg) */

ATbool ADT_isValidModules(ADT_Modules arg)
{
  if (ADT_isModulesEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT_isModulesSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isModulesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isModulesEmpty(ADT_Modules arg) */

inline ATbool ADT_isModulesEmpty(ADT_Modules arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternModulesEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT_isModulesSingle(ADT_Modules arg) */

inline ATbool ADT_isModulesSingle(ADT_Modules arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternModulesSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isModulesMany(ADT_Modules arg) */

inline ATbool ADT_isModulesMany(ADT_Modules arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternModulesMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasModulesHead(ADT_Modules arg) */

ATbool ADT_hasModulesHead(ADT_Modules arg)
{
  if (ADT_isModulesSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isModulesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasModulesTail(ADT_Modules arg) */

ATbool ADT_hasModulesTail(ADT_Modules arg)
{
  if (ADT_isModulesMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_Module ADT_getModulesHead(ADT_Modules arg) */

ADT_Module ADT_getModulesHead(ADT_Modules arg)
{
  if (ADT_isModulesSingle(arg)) {
    return (ADT_Module)ATgetFirst((ATermList)arg);
  }
  else 
    return (ADT_Module)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Modules ADT_getModulesTail(ADT_Modules arg) */

ADT_Modules ADT_getModulesTail(ADT_Modules arg)
{
  
    return (ADT_Modules)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Modules ADT_setModulesHead(ADT_Modules arg, ADT_Module head) */

ADT_Modules ADT_setModulesHead(ADT_Modules arg, ADT_Module head)
{
  if (ADT_isModulesSingle(arg)) {
    return (ADT_Modules)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (ADT_isModulesMany(arg)) {
    return (ADT_Modules)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Modules has no Head: %t\n", arg);
  return (ADT_Modules)NULL;
}

/*}}}  */
/*{{{  ADT_Modules ADT_setModulesTail(ADT_Modules arg, ADT_Modules tail) */

ADT_Modules ADT_setModulesTail(ADT_Modules arg, ADT_Modules tail)
{
  if (ADT_isModulesMany(arg)) {
    return (ADT_Modules)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Modules has no Tail: %t\n", arg);
  return (ADT_Modules)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Module accessors */

/*{{{  ATbool ADT_isValidModule(ADT_Module arg) */

ATbool ADT_isValidModule(ADT_Module arg)
{
  if (ADT_isModuleModulentry(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isModuleModulentry(ADT_Module arg) */

inline ATbool ADT_isModuleModulentry(ADT_Module arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternModuleModulentry, NULL, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADT_hasModuleModulename(ADT_Module arg) */

ATbool ADT_hasModuleModulename(ADT_Module arg)
{
  if (ADT_isModuleModulentry(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasModuleImports(ADT_Module arg) */

ATbool ADT_hasModuleImports(ADT_Module arg)
{
  if (ADT_isModuleModulentry(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasModuleSorts(ADT_Module arg) */

ATbool ADT_hasModuleSorts(ADT_Module arg)
{
  if (ADT_isModuleModulentry(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasModuleEntries(ADT_Module arg) */

ATbool ADT_hasModuleEntries(ADT_Module arg)
{
  if (ADT_isModuleModulentry(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_ModuleName ADT_getModuleModulename(ADT_Module arg) */

ADT_ModuleName ADT_getModuleModulename(ADT_Module arg)
{
  
    return (ADT_ModuleName)ATgetArgument((ATermAppl)arg, 0);
}

/*}}}  */
/*{{{  ADT_Imports ADT_getModuleImports(ADT_Module arg) */

ADT_Imports ADT_getModuleImports(ADT_Module arg)
{
  
    return (ADT_Imports)ATgetArgument((ATermAppl)arg, 1);
}

/*}}}  */
/*{{{  ADT_Sorts ADT_getModuleSorts(ADT_Module arg) */

ADT_Sorts ADT_getModuleSorts(ADT_Module arg)
{
  
    return (ADT_Sorts)ATgetArgument((ATermAppl)arg, 2);
}

/*}}}  */
/*{{{  ADT_Entries ADT_getModuleEntries(ADT_Module arg) */

ADT_Entries ADT_getModuleEntries(ADT_Module arg)
{
  
    return (ADT_Entries)ATgetArgument((ATermAppl)arg, 3);
}

/*}}}  */
/*{{{  ADT_Module ADT_setModuleModulename(ADT_Module arg, ADT_ModuleName modulename) */

ADT_Module ADT_setModuleModulename(ADT_Module arg, ADT_ModuleName modulename)
{
  if (ADT_isModuleModulentry(arg)) {
    return (ADT_Module)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) modulename), 0);
  }

  ATabort("Module has no Modulename: %t\n", arg);
  return (ADT_Module)NULL;
}

/*}}}  */
/*{{{  ADT_Module ADT_setModuleImports(ADT_Module arg, ADT_Imports imports) */

ADT_Module ADT_setModuleImports(ADT_Module arg, ADT_Imports imports)
{
  if (ADT_isModuleModulentry(arg)) {
    return (ADT_Module)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) imports), 1);
  }

  ATabort("Module has no Imports: %t\n", arg);
  return (ADT_Module)NULL;
}

/*}}}  */
/*{{{  ADT_Module ADT_setModuleSorts(ADT_Module arg, ADT_Sorts sorts) */

ADT_Module ADT_setModuleSorts(ADT_Module arg, ADT_Sorts sorts)
{
  if (ADT_isModuleModulentry(arg)) {
    return (ADT_Module)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) sorts), 2);
  }

  ATabort("Module has no Sorts: %t\n", arg);
  return (ADT_Module)NULL;
}

/*}}}  */
/*{{{  ADT_Module ADT_setModuleEntries(ADT_Module arg, ADT_Entries entries) */

ADT_Module ADT_setModuleEntries(ADT_Module arg, ADT_Entries entries)
{
  if (ADT_isModuleModulentry(arg)) {
    return (ADT_Module)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) entries), 3);
  }

  ATabort("Module has no Entries: %t\n", arg);
  return (ADT_Module)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Imports accessors */

/*{{{  ATbool ADT_isValidImports(ADT_Imports arg) */

ATbool ADT_isValidImports(ADT_Imports arg)
{
  if (ADT_isImportsEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT_isImportsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isImportsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isImportsEmpty(ADT_Imports arg) */

inline ATbool ADT_isImportsEmpty(ADT_Imports arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternImportsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT_isImportsSingle(ADT_Imports arg) */

inline ATbool ADT_isImportsSingle(ADT_Imports arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternImportsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isImportsMany(ADT_Imports arg) */

inline ATbool ADT_isImportsMany(ADT_Imports arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternImportsMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasImportsHead(ADT_Imports arg) */

ATbool ADT_hasImportsHead(ADT_Imports arg)
{
  if (ADT_isImportsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isImportsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasImportsTail(ADT_Imports arg) */

ATbool ADT_hasImportsTail(ADT_Imports arg)
{
  if (ADT_isImportsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_ModuleName ADT_getImportsHead(ADT_Imports arg) */

ADT_ModuleName ADT_getImportsHead(ADT_Imports arg)
{
  if (ADT_isImportsSingle(arg)) {
    return (ADT_ModuleName)ATgetFirst((ATermList)arg);
  }
  else 
    return (ADT_ModuleName)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Imports ADT_getImportsTail(ADT_Imports arg) */

ADT_Imports ADT_getImportsTail(ADT_Imports arg)
{
  
    return (ADT_Imports)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Imports ADT_setImportsHead(ADT_Imports arg, ADT_ModuleName head) */

ADT_Imports ADT_setImportsHead(ADT_Imports arg, ADT_ModuleName head)
{
  if (ADT_isImportsSingle(arg)) {
    return (ADT_Imports)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (ADT_isImportsMany(arg)) {
    return (ADT_Imports)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Imports has no Head: %t\n", arg);
  return (ADT_Imports)NULL;
}

/*}}}  */
/*{{{  ADT_Imports ADT_setImportsTail(ADT_Imports arg, ADT_Imports tail) */

ADT_Imports ADT_setImportsTail(ADT_Imports arg, ADT_Imports tail)
{
  if (ADT_isImportsMany(arg)) {
    return (ADT_Imports)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Imports has no Tail: %t\n", arg);
  return (ADT_Imports)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Type accessors */

/*{{{  ATbool ADT_isValidType(ADT_Type arg) */

ATbool ADT_isValidType(ADT_Type arg)
{
  if (ADT_isTypeType(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isTypeType(ADT_Type arg) */

inline ATbool ADT_isTypeType(ADT_Type arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternTypeType, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADT_hasTypeName(ADT_Type arg) */

ATbool ADT_hasTypeName(ADT_Type arg)
{
  if (ADT_isTypeType(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* ADT_getTypeName(ADT_Type arg) */

char* ADT_getTypeName(ADT_Type arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATgetArgument((ATermAppl)arg, 0)));
}

/*}}}  */
/*{{{  ADT_Type ADT_setTypeName(ADT_Type arg, const char* name) */

ADT_Type ADT_setTypeName(ADT_Type arg, const char* name)
{
  if (ADT_isTypeType(arg)) {
    return (ADT_Type)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(name, 0, ATtrue))), 0);
  }

  ATabort("Type has no Name: %t\n", arg);
  return (ADT_Type)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_Sorts accessors */

/*{{{  ATbool ADT_isValidSorts(ADT_Sorts arg) */

ATbool ADT_isValidSorts(ADT_Sorts arg)
{
  if (ADT_isSortsEmpty(arg)) {
    return ATtrue;
  }
  else if (ADT_isSortsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isSortsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSortsEmpty(ADT_Sorts arg) */

inline ATbool ADT_isSortsEmpty(ADT_Sorts arg)
{
  if (!ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternSortsEmpty));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  inline ATbool ADT_isSortsSingle(ADT_Sorts arg) */

inline ATbool ADT_isSortsSingle(ADT_Sorts arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternSortsSingle, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  inline ATbool ADT_isSortsMany(ADT_Sorts arg) */

inline ATbool ADT_isSortsMany(ADT_Sorts arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
  {
    static ATerm last_arg = NULL;
    static int last_gc = -1;
    static ATbool last_result;

    assert(arg != NULL);

    if (last_gc != ATgetGCCount() || (ATerm)arg != last_arg) {
      last_arg = (ATerm)arg;
      last_result = ATmatchTerm((ATerm)arg, ADT_patternSortsMany, NULL, NULL);
      last_gc = ATgetGCCount();
    }

    return last_result;
  }
}

/*}}}  */
/*{{{  ATbool ADT_hasSortsHead(ADT_Sorts arg) */

ATbool ADT_hasSortsHead(ADT_Sorts arg)
{
  if (ADT_isSortsSingle(arg)) {
    return ATtrue;
  }
  else if (ADT_isSortsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ATbool ADT_hasSortsTail(ADT_Sorts arg) */

ATbool ADT_hasSortsTail(ADT_Sorts arg)
{
  if (ADT_isSortsMany(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADT_Type ADT_getSortsHead(ADT_Sorts arg) */

ADT_Type ADT_getSortsHead(ADT_Sorts arg)
{
  if (ADT_isSortsSingle(arg)) {
    return (ADT_Type)ATgetFirst((ATermList)arg);
  }
  else 
    return (ADT_Type)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Sorts ADT_getSortsTail(ADT_Sorts arg) */

ADT_Sorts ADT_getSortsTail(ADT_Sorts arg)
{
  
    return (ADT_Sorts)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADT_Sorts ADT_setSortsHead(ADT_Sorts arg, ADT_Type head) */

ADT_Sorts ADT_setSortsHead(ADT_Sorts arg, ADT_Type head)
{
  if (ADT_isSortsSingle(arg)) {
    return (ADT_Sorts)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }
  else if (ADT_isSortsMany(arg)) {
    return (ADT_Sorts)ATreplace((ATermList)arg, (ATerm)((ATerm) head), 0);
  }

  ATabort("Sorts has no Head: %t\n", arg);
  return (ADT_Sorts)NULL;
}

/*}}}  */
/*{{{  ADT_Sorts ADT_setSortsTail(ADT_Sorts arg, ADT_Sorts tail) */

ADT_Sorts ADT_setSortsTail(ADT_Sorts arg, ADT_Sorts tail)
{
  if (ADT_isSortsMany(arg)) {
    return (ADT_Sorts)ATreplaceTail((ATermList)arg, (ATermList)((ATerm) tail), 1);
  }

  ATabort("Sorts has no Tail: %t\n", arg);
  return (ADT_Sorts)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADT_ModuleName accessors */

/*{{{  ATbool ADT_isValidModuleName(ADT_ModuleName arg) */

ATbool ADT_isValidModuleName(ADT_ModuleName arg)
{
  if (ADT_isModuleNameName(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADT_isModuleNameName(ADT_ModuleName arg) */

inline ATbool ADT_isModuleNameName(ADT_ModuleName arg)
{
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADT_patternModuleNameName, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADT_hasModuleNameName(ADT_ModuleName arg) */

ATbool ADT_hasModuleNameName(ADT_ModuleName arg)
{
  if (ADT_isModuleNameName(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  char* ADT_getModuleNameName(ADT_ModuleName arg) */

char* ADT_getModuleNameName(ADT_ModuleName arg)
{
  
    return (char*)ATgetName(ATgetAFun((ATermAppl) ATgetArgument((ATermAppl)arg, 0)));
}

/*}}}  */
/*{{{  ADT_ModuleName ADT_setModuleNameName(ADT_ModuleName arg, const char* name) */

ADT_ModuleName ADT_setModuleNameName(ADT_ModuleName arg, const char* name)
{
  if (ADT_isModuleNameName(arg)) {
    return (ADT_ModuleName)ATsetArgument((ATermAppl)arg, (ATerm)((ATerm) (ATerm) ATmakeAppl(ATmakeAFun(name, 0, ATtrue))), 0);
  }

  ATabort("ModuleName has no Name: %t\n", arg);
  return (ADT_ModuleName)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  ADT_Entries ADT_visitEntries(ADT_Entries arg, ADT_Entry (*acceptHead)(ADT_Entry)) */

ADT_Entries ADT_visitEntries(ADT_Entries arg, ADT_Entry (*acceptHead)(ADT_Entry))
{
  if (ADT_isEntriesEmpty(arg)) {
    return ADT_makeEntriesEmpty();
  }
  if (ADT_isEntriesSingle(arg)) {
    return ADT_makeEntriesSingle(
        acceptHead ? acceptHead(ADT_getEntriesHead(arg)) : ADT_getEntriesHead(arg));
  }
  if (ADT_isEntriesMany(arg)) {
    return ADT_makeEntriesMany(
        acceptHead ? acceptHead(ADT_getEntriesHead(arg)) : ADT_getEntriesHead(arg),
        ADT_visitEntries(ADT_getEntriesTail(arg), acceptHead));
  }
  ATabort("not a Entries: %t\n", arg);
  return (ADT_Entries)NULL;
}

/*}}}  */
/*{{{  ADT_Entry ADT_visitEntry(ADT_Entry arg, ATerm (*acceptSort)(ATerm), ATerm (*acceptAlternative)(ATerm), ATerm (*acceptTermPattern)(ATerm), ATerm (*acceptElemSort)(ATerm), ATerm (*acceptOpname)(ATerm), ADT_Separators (*acceptSeparators)(ADT_Separators)) */

ADT_Entry ADT_visitEntry(ADT_Entry arg, ATerm (*acceptSort)(ATerm), ATerm (*acceptAlternative)(ATerm), ATerm (*acceptTermPattern)(ATerm), ATerm (*acceptElemSort)(ATerm), ATerm (*acceptOpname)(ATerm), ADT_Separators (*acceptSeparators)(ADT_Separators))
{
  if (ADT_isEntryConstructor(arg)) {
    return ADT_makeEntryConstructor(
        acceptSort ? acceptSort(ADT_getEntrySort(arg)) : ADT_getEntrySort(arg),
        acceptAlternative ? acceptAlternative(ADT_getEntryAlternative(arg)) : ADT_getEntryAlternative(arg),
        acceptTermPattern ? acceptTermPattern(ADT_getEntryTermPattern(arg)) : ADT_getEntryTermPattern(arg));
  }
  if (ADT_isEntryList(arg)) {
    return ADT_makeEntryList(
        acceptSort ? acceptSort(ADT_getEntrySort(arg)) : ADT_getEntrySort(arg),
        acceptElemSort ? acceptElemSort(ADT_getEntryElemSort(arg)) : ADT_getEntryElemSort(arg));
  }
  if (ADT_isEntryNamedList(arg)) {
    return ADT_makeEntryNamedList(
        acceptOpname ? acceptOpname(ADT_getEntryOpname(arg)) : ADT_getEntryOpname(arg),
        acceptSort ? acceptSort(ADT_getEntrySort(arg)) : ADT_getEntrySort(arg),
        acceptElemSort ? acceptElemSort(ADT_getEntryElemSort(arg)) : ADT_getEntryElemSort(arg));
  }
  if (ADT_isEntrySeparatedList(arg)) {
    return ADT_makeEntrySeparatedList(
        acceptSort ? acceptSort(ADT_getEntrySort(arg)) : ADT_getEntrySort(arg),
        acceptElemSort ? acceptElemSort(ADT_getEntryElemSort(arg)) : ADT_getEntryElemSort(arg),
        acceptSeparators ? acceptSeparators(ADT_getEntrySeparators(arg)) : ADT_getEntrySeparators(arg));
  }
  ATabort("not a Entry: %t\n", arg);
  return (ADT_Entry)NULL;
}

/*}}}  */
/*{{{  ADT_Separators ADT_visitSeparators(ADT_Separators arg, ADT_Separator (*acceptHead)(ADT_Separator)) */

ADT_Separators ADT_visitSeparators(ADT_Separators arg, ADT_Separator (*acceptHead)(ADT_Separator))
{
  if (ADT_isSeparatorsEmpty(arg)) {
    return ADT_makeSeparatorsEmpty();
  }
  if (ADT_isSeparatorsSingle(arg)) {
    return ADT_makeSeparatorsSingle(
        acceptHead ? acceptHead(ADT_getSeparatorsHead(arg)) : ADT_getSeparatorsHead(arg));
  }
  if (ADT_isSeparatorsMany(arg)) {
    return ADT_makeSeparatorsMany(
        acceptHead ? acceptHead(ADT_getSeparatorsHead(arg)) : ADT_getSeparatorsHead(arg),
        ADT_visitSeparators(ADT_getSeparatorsTail(arg), acceptHead));
  }
  ATabort("not a Separators: %t\n", arg);
  return (ADT_Separators)NULL;
}

/*}}}  */
/*{{{  ADT_Separator ADT_visitSeparator(ADT_Separator arg, ATerm (*acceptTermPattern)(ATerm)) */

ADT_Separator ADT_visitSeparator(ADT_Separator arg, ATerm (*acceptTermPattern)(ATerm))
{
  if (ADT_isSeparatorDefault(arg)) {
    return ADT_makeSeparatorDefault(
        acceptTermPattern ? acceptTermPattern(ADT_getSeparatorTermPattern(arg)) : ADT_getSeparatorTermPattern(arg));
  }
  ATabort("not a Separator: %t\n", arg);
  return (ADT_Separator)NULL;
}

/*}}}  */
/*{{{  ADT_Modules ADT_visitModules(ADT_Modules arg, ADT_Module (*acceptHead)(ADT_Module)) */

ADT_Modules ADT_visitModules(ADT_Modules arg, ADT_Module (*acceptHead)(ADT_Module))
{
  if (ADT_isModulesEmpty(arg)) {
    return ADT_makeModulesEmpty();
  }
  if (ADT_isModulesSingle(arg)) {
    return ADT_makeModulesSingle(
        acceptHead ? acceptHead(ADT_getModulesHead(arg)) : ADT_getModulesHead(arg));
  }
  if (ADT_isModulesMany(arg)) {
    return ADT_makeModulesMany(
        acceptHead ? acceptHead(ADT_getModulesHead(arg)) : ADT_getModulesHead(arg),
        ADT_visitModules(ADT_getModulesTail(arg), acceptHead));
  }
  ATabort("not a Modules: %t\n", arg);
  return (ADT_Modules)NULL;
}

/*}}}  */
/*{{{  ADT_Module ADT_visitModule(ADT_Module arg, ADT_ModuleName (*acceptModulename)(ADT_ModuleName), ADT_Imports (*acceptImports)(ADT_Imports), ADT_Sorts (*acceptSorts)(ADT_Sorts), ADT_Entries (*acceptEntries)(ADT_Entries)) */

ADT_Module ADT_visitModule(ADT_Module arg, ADT_ModuleName (*acceptModulename)(ADT_ModuleName), ADT_Imports (*acceptImports)(ADT_Imports), ADT_Sorts (*acceptSorts)(ADT_Sorts), ADT_Entries (*acceptEntries)(ADT_Entries))
{
  if (ADT_isModuleModulentry(arg)) {
    return ADT_makeModuleModulentry(
        acceptModulename ? acceptModulename(ADT_getModuleModulename(arg)) : ADT_getModuleModulename(arg),
        acceptImports ? acceptImports(ADT_getModuleImports(arg)) : ADT_getModuleImports(arg),
        acceptSorts ? acceptSorts(ADT_getModuleSorts(arg)) : ADT_getModuleSorts(arg),
        acceptEntries ? acceptEntries(ADT_getModuleEntries(arg)) : ADT_getModuleEntries(arg));
  }
  ATabort("not a Module: %t\n", arg);
  return (ADT_Module)NULL;
}

/*}}}  */
/*{{{  ADT_Imports ADT_visitImports(ADT_Imports arg, ADT_ModuleName (*acceptHead)(ADT_ModuleName)) */

ADT_Imports ADT_visitImports(ADT_Imports arg, ADT_ModuleName (*acceptHead)(ADT_ModuleName))
{
  if (ADT_isImportsEmpty(arg)) {
    return ADT_makeImportsEmpty();
  }
  if (ADT_isImportsSingle(arg)) {
    return ADT_makeImportsSingle(
        acceptHead ? acceptHead(ADT_getImportsHead(arg)) : ADT_getImportsHead(arg));
  }
  if (ADT_isImportsMany(arg)) {
    return ADT_makeImportsMany(
        acceptHead ? acceptHead(ADT_getImportsHead(arg)) : ADT_getImportsHead(arg),
        ADT_visitImports(ADT_getImportsTail(arg), acceptHead));
  }
  ATabort("not a Imports: %t\n", arg);
  return (ADT_Imports)NULL;
}

/*}}}  */
/*{{{  ADT_Type ADT_visitType(ADT_Type arg, char* (*acceptName)(char*)) */

ADT_Type ADT_visitType(ADT_Type arg, char* (*acceptName)(char*))
{
  if (ADT_isTypeType(arg)) {
    return ADT_makeTypeType(
        acceptName ? acceptName(ADT_getTypeName(arg)) : ADT_getTypeName(arg));
  }
  ATabort("not a Type: %t\n", arg);
  return (ADT_Type)NULL;
}

/*}}}  */
/*{{{  ADT_Sorts ADT_visitSorts(ADT_Sorts arg, ADT_Type (*acceptHead)(ADT_Type)) */

ADT_Sorts ADT_visitSorts(ADT_Sorts arg, ADT_Type (*acceptHead)(ADT_Type))
{
  if (ADT_isSortsEmpty(arg)) {
    return ADT_makeSortsEmpty();
  }
  if (ADT_isSortsSingle(arg)) {
    return ADT_makeSortsSingle(
        acceptHead ? acceptHead(ADT_getSortsHead(arg)) : ADT_getSortsHead(arg));
  }
  if (ADT_isSortsMany(arg)) {
    return ADT_makeSortsMany(
        acceptHead ? acceptHead(ADT_getSortsHead(arg)) : ADT_getSortsHead(arg),
        ADT_visitSorts(ADT_getSortsTail(arg), acceptHead));
  }
  ATabort("not a Sorts: %t\n", arg);
  return (ADT_Sorts)NULL;
}

/*}}}  */
/*{{{  ADT_ModuleName ADT_visitModuleName(ADT_ModuleName arg, char* (*acceptName)(char*)) */

ADT_ModuleName ADT_visitModuleName(ADT_ModuleName arg, char* (*acceptName)(char*))
{
  if (ADT_isModuleNameName(arg)) {
    return ADT_makeModuleNameName(
        acceptName ? acceptName(ADT_getModuleNameName(arg)) : ADT_getModuleNameName(arg));
  }
  ATabort("not a ModuleName: %t\n", arg);
  return (ADT_ModuleName)NULL;
}

/*}}}  */

/*}}}  */
